% Write sensitivity results (NROSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Noise Reduced Reaction-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%    MC COMPUTATION WITH DIMENSIONALITY REDUCTION      %%%%%%%%%%
%%%%%%%%%%%              Authors: Zhang/Goutsias                 %%%%%%%%%%
%%%%%%%%%%%             Last Modified: 09/18/2010                %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write SESIs and JESIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% TIMING

fid = fopen('..\results\reports\timing_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'NR MC-BASED REACTION RESULTS (TIMING)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',rlambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nReaction\tSESI\t\tJESI\t\tClassification\n');
for m = 1:nR
    Sens = [reac(m);sigmat(m);etat(m)];
    if sigmat(m) <= theta && etat(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmat(m) > theta && etat(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmat(m) <= theta && etat(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmat(m) > theta && etat(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);

% DURATION

fid = fopen('..\results\reports\duration_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'NR MC-BASED REACTION RESULTS (DURATION)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',rlambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nReaction\tSESI\t\tJESI\t\tClassification\n');
for m = 1:nR
    Sens = [reac(m);sigmad(m);etad(m)];
    if sigmad(m) <= theta && etad(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmad(m) > theta && etad(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmad(m) <= theta && etad(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmad(m) > theta && etad(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);

% STRENGTH

fid = fopen('..\results\reports\strength_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'NR MC-BASED REACTION RESULTS (STRENGTH)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',rlambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nReaction\tSESI\t\tJESI\t\tClassification\n');
for m = 1:nR
    Sens = [reac(m);sigmas(m);etas(m)];
    if sigmas(m) <= theta && etas(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmas(m) > theta && etas(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmas(m) <= theta && etas(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmas(m) > theta && etas(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);

% RATIO

fid = fopen('..\results\reports\ratio_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'NR MC-BASED REACTION RESULTS (RATIO)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',rlambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nReaction\tSESI\t\tJESI\t\tClassification\n');
for m = 1:nR
    Sens = [reac(m);sigmar(m);etar(m)];
    if sigmar(m) <= theta && etar(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmar(m) > theta && etar(m) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmar(m) <= theta && etar(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmar(m) > theta && etar(m) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);
