% MAIN FILE (NROSA-OHA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%  Noise Reduced Reaction-Oriented Sensitivity Analysis   %%%%%%%%%
%%%%%%%%%                   OHA COMPUTATION                       %%%%%%%%%
%%%%%%%%%               Authors: Zhang/Goutsias                   %%%%%%%%%
%%%%%%%%%              Last Modified: 09/17/2010                  %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;              % clear all parameters
close all;              % close all files

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
 
parameters_EGFR;        % system parameters
x0_vec_EGFR;            % initial molecular numbers

rlambda = 0.4;          % size of biochemical perturbations
sigma   = 1.0;          % size of experimental perurbations
eps     = 0.4;          % classification threshold 
 
A       = zeros((2*M),(M+N));
vstoich = (-stoich.*(stoich < 0))';

% Michaelis-Menten kinetics

vstoich(7,5) = 0;
vstoich(8,4) = 0;
vstoich(8,5) = -1;
vstoich(15,9) = 0;
vstoich(16,6) = 0;
vstoich(16,9) = -1;
vstoich(31,18) = 0;
vstoich(32,15) = 0;
vstoich(32,18) = -1;
A(:,(M+1):(M+N)) = vstoich;
for m = 1:M
    if (m == 4) || (m == 8) || (m == 16) % Michaelis-Menten kinetics
        A((2*m-1),m) = -1;
    else
        A((2*m-1),m) = -1;               % Mass Action kinetics
        A((2*m),m) = -1;
    end
end

load V.mat
 
P = size(V,2);          % number of experimental factors
reac(1:M) = 1:M;
L      = 8*(M+P)^2;     % number of MC-LHS samples
time   = 240;           % simulation time (sec)

iode   = 1;             % iode=0 use of Matlab ODE solver (slow)
                        % iode=1 use of custom ODE solver (fast)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

durtol  = 0.05;                          % duration tolerance
tresol  = 0.2;                           % time resolution (in min)
tpoints = (0:tresol:time);               % Time points for ODE integration
tmax    = size(tpoints,2);               % Number of time points
odetol  = 1e-06;                         % tolerance for ODE solver
options = odeset('RelTol',odetol,'AbsTol',odetol);  % options for Matlab ODE solver
ndata   = time/tresol;
timings = [0,time,ndata];

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate regression coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

regressit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Estimate variances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

varest;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate SIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

senseit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write SI results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

writeit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Save workspace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

save('..\results\workspace\nrosa_oha.mat');