% MC-LHS variance estimation (NSOSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Noise Reduced Reaction-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%    MC COMPUTATION WITH DIMENSIONALITY REDUCTION      %%%%%%%%%%
%%%%%%%%%%%              Authors: Zhang/Goutsias                 %%%%%%%%%%
%%%%%%%%%%%             Last Modified: 09/20/2010                %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initializations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

vstoich = (-stoich.*(stoich < 0))';

J     = N + P;
perm1 = repmat(0,[J,L]);
perm2 = repmat(0,[J,L]);

varmTT(1:nS) = 0;
varmDD(1:nS) = 0;
varmSS(1:nS) = 0;
varmRR(1:nS) = 0;

varmzTT(1:nS) = 0;
varmzDD(1:nS) = 0;
varmzSS(1:nS) = 0;
varmzRR(1:nS) = 0;

varfTT(1:nS) = 0;
varfDD(1:nS) = 0;
varfSS(1:nS) = 0;
varfRR(1:nS) = 0;

varvTT(1:nS) = 0;
varvDD(1:nS) = 0;
varvSS(1:nS) = 0;
varvRR(1:nS) = 0;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MC-LHS iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Determine LHS permutations
%
rand('state',sum(100*clock));     %% Reset state of random number generator
for j = 1:J
    perm1(j,:) = randperm(L);
    perm2(j,:) = randperm(L);
end

for iter = 1:L
    
    iterleft = L+1-iter;
    fprintf('Remaining MC-LHS iterations: %6.0f\n',iterleft)
    
%
% Calculate Latin hypercube samples
%

    uniform = rand(J,1);
    index1  = (perm1(:,iter)-1+uniform)./L;
    ZS1     = repmat(0,[P,1]);
    ZS1     = sigma*(-sqrt(2)*erfcinv(2*index1(1:P)));       % inverse normal CDF
    vzs1    = V*ZS1;
    kappa1  = kappa.*exp(vzs1);
    
    YS1     = slambda*(-sqrt(2)*erfcinv(2*index1((P+1):J))); % inverse normal CDF
    svyn1   = vstoich*YS1;
    % Michaelis-Menten kinetics
    svyn1(7) = 0;
    svyn1(8) = -YS1(5);
    svyn1(15) = 0;
    svyn1(16) = -YS1(9);
    svyn1(31) = 0;
    svyn1(32) = -YS1(18);
    kappa1  = kappa1.*exp(svyn1);
    
    uniform = rand(J,1);
    index2  = (perm2(:,iter)-1+uniform)./L;
    ZS2     = repmat(0,[P,1]);
    ZS2     = sigma*(-sqrt(2)*erfcinv(2*index2(1:P)));       % inverse normal CDF
    vzs2    = V*ZS2;
    kappa2  = kappa.*exp(vzs2);
    
    YS2     = slambda*(-sqrt(2)*erfcinv(2*index2((P+1):J))); % inverse normal CDF
    svyn2   = vstoich*YS2;
    % Michaelis-Menten kinetics
    svyn2(7) = 0;
    svyn2(8) = -YS2(5);
    svyn2(15) = 0;
    svyn2(16) = -YS2(9);
    svyn2(31) = 0;
    svyn2(32) = -YS2(18);
    kappa2  = kappa2.*exp(svyn2);
    
    kappa3  = kappa.*exp(vzs1);
    kappa3  = kappa3.*exp(svyn2);
    
    kappa4  = kappa.*exp(vzs2);
    kappa4  = kappa4.*exp(svyn1);
    
    YS5(1,:) = YS2';
    YS6(1,:) = YS1';
    YS7(1,:) = YS1';
    YS8(1,:) = YS2';

    for n = 1:(N+1-nS)
        YS5(1,nspe(n)) = YS1(nspe(n));
        YS5(1,nspe(n)) = YS1(nspe(n));
        YS6(1,nspe(n)) = YS2(nspe(n));
        YS6(1,nspe(n)) = YS2(nspe(n));
        YS7(1,nspe(n)) = YS2(nspe(n));
        YS7(1,nspe(n)) = YS2(nspe(n));
        YS8(1,nspe(n)) = YS1(nspe(n));
        YS8(1,nspe(n)) = YS1(nspe(n));
    end

    for n = 2:nS
        YS5(n,:) = YS2';
        YS6(n,:) = YS1';
        YS7(n,:) = YS1';
        YS8(n,:) = YS2';

        YS5(n,spec(n)) = YS1(spec(n));
        YS6(n,spec(n)) = YS2(spec(n));
        YS7(n,spec(n)) = YS2(spec(n));
        YS8(n,spec(n)) = YS1(spec(n));
    end

%
% Calculate response characteristics (timing, duration, and strength)
%    
% For kappa1

    if iode == 1
        try
            [t,x] = egfr(kappa1,x0,[odetol,odetol],timings);
        catch
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa1,stoich);
        end
    else
        [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa1,stoich);
    end
    
    [TT1,DD1,SS1,RR1] = featureit_EGFR(x,t,tresol,durtol,odetol);
    
% For kappa2

    if iode == 1
        try
            [t,x] = egfr(kappa2,x0,[odetol,odetol],timings);
        catch
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa2,stoich);
        end
    else
        [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa2,stoich);
    end
    
    [TT2,DD2,SS2,RR2] = featureit_EGFR(x,t,tresol,durtol,odetol);
    
% For kappa3

    if iode == 1
        try
            [t,x] = egfr(kappa3,x0,[odetol,odetol],timings);
        catch
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa3,stoich);
        end
    else
        [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,N,kappa3,stoich);
    end
    
    [TT3,DD3,SS3,RR3] = featureit_EGFR(x,t,tresol,durtol,odetol);
   
    
% For kappa4

    if iode == 1
        try
            [t,x] = egfr(kappa4,x0,[odetol,odetol],timings);
        catch
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa4,stoich);
        end
    else
        [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,N,kappa4,stoich);
    end
    
    [TT4,DD4,SS4,RR4] = featureit_EGFR(x,t,tresol,durtol,odetol);
    
% For kappa5 to kappa8

    for n = 1:nS
        
        kappa5  = kappa.*exp(vzs2);
        svyn   = vstoich*YS5(n,:)';
        % Michaelis-Menten kinetics
        svyn(7) = 0;
        svyn(8) = -YS5(n,5);
        svyn(15)= 0;
        svyn(16)= -YS5(n,9);
        svyn(31)= 0;
        svyn(32)= -YS5(n,18);
        kappa5  = kappa5.*exp(svyn);
        
        if iode == 1
            try
                [t,x] = egfr(kappa5,x0,[odetol,odetol],timings);
            catch
                [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa5,stoich);
            end
        else
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa5,stoich);
        end
        
        [TT5(n),DD5(n),SS5(n),RR5(n)] = featureit_EGFR(x,t,tresol,durtol,odetol);

        kappa6  = kappa.*exp(vzs1);
        svyn   = vstoich*YS6(n,:)';
        % Michaelis-Menten kinetics
        svyn(7) = 0;
        svyn(8) = -YS6(n,5);
        svyn(15)= 0;
        svyn(16)= -YS6(n,9);
        svyn(31)= 0;
        svyn(32)= -YS6(n,18);
        kappa6  = kappa6.*exp(svyn);
        
        if iode == 1
            try
                [t,x] = egfr(kappa6,x0,[odetol,odetol],timings);
            catch
                [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa6,stoich);
            end
        else
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa6,stoich);
        end
        
        [TT6(n),DD6(n),SS6(n),RR6(n)] = featureit_EGFR(x,t,tresol,durtol,odetol);
        
        kappa7  = kappa.*exp(vzs2);
        svyn   = vstoich*YS7(n,:)';
        % Michaelis-Menten kinetics
        svyn(7) = 0;
        svyn(8) = -YS7(n,5);
        svyn(15)= 0;
        svyn(16)= -YS7(n,9);
        svyn(31)= 0;
        svyn(32)= -YS7(n,18);
        kappa7  = kappa7.*exp(svyn);
        
        if iode == 1
            try
                [t,x] = egfr(kappa7,x0,[odetol,odetol],timings);
            catch
                [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa7,stoich);
            end
        else
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa7,stoich);
        end
        
        [TT7(n),DD7(n),SS7(n),RR7(n)] = featureit_EGFR(x,t,tresol,durtol,odetol);
        
        kappa8  = kappa.*exp(vzs1);
        svyn   = vstoich*YS8(n,:)';
        % Michaelis-Menten kinetics
        svyn(7) = 0;
        svyn(8) = -YS8(n,5);
        svyn(15)= 0;
        svyn(16)= -YS8(n,9);
        svyn(31)= 0;
        svyn(32)= -YS8(n,18);
        kappa8  = kappa8.*exp(svyn);
        
        if iode == 1
            try
                [t,x] = egfr(kappa8,x0,[odetol,odetol],timings);
            catch
                [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa8,stoich);
            end
        else
            [t,x] = ode15s(@netode_EGFR,tpoints,x0,options,kappa8,stoich);
        end
        
        [TT8(n),DD8(n),SS8(n),RR8(n)] = featureit_EGFR(x,t,tresol,durtol,odetol);
        
    end
    
%
% Update variance calculations
%    
    varmTT = varmTT + (TT1-TT2)^2 + (TT3-TT4)^2 + (TT5-TT6).^2 + (TT7-TT8).^2;
    varmDD = varmDD + (DD1-DD2)^2 + (DD3-DD4)^2 + (DD5-DD6).^2 + (DD7-DD8).^2;
    varmSS = varmSS + (SS1-SS2)^2 + (SS3-SS4)^2 + (SS5-SS6).^2 + (SS7-SS8).^2;
    varmRR = varmRR + (RR1-RR2)^2 + (RR3-RR4)^2 + (RR5-RR6).^2 + (RR7-RR8).^2;
    
    varmzTT = varmzTT + (TT1-TT4).*(TT3-TT2) + (TT5-TT8).*(TT7-TT6);
    varmzDD = varmzDD + (DD1-DD4).*(DD3-DD2) + (DD5-DD8).*(DD7-DD6);
    varmzSS = varmzSS + (SS1-SS4).*(SS3-SS2) + (SS5-SS8).*(SS7-SS6);
    varmzRR = varmzRR + (RR1-RR4).*(RR3-RR2) + (RR5-RR8).*(RR7-RR6);
    
    varfTT = varfTT + (TT1-TT7).*(TT8-TT2) + (TT3-TT5).*(TT6-TT4);                
    varfDD = varfDD + (DD1-DD7).*(DD8-DD2) + (DD3-DD5).*(DD6-DD4);
    varfSS = varfSS + (SS1-SS7).*(SS8-SS2) + (SS3-SS5).*(SS6-SS4);
    varfRR = varfRR + (RR1-RR7).*(RR8-RR2) + (RR3-RR5).*(RR6-RR4);

    varvTT = varvTT + (TT7-TT3).*(TT4-TT8) + (TT1-TT5).*(TT6-TT2);
    varvDD = varvDD + (DD7-DD3).*(DD4-DD8) + (DD1-DD5).*(DD6-DD2);
    varvSS = varvSS + (SS7-SS3).*(SS4-SS8) + (SS1-SS5).*(SS6-SS2);
    varvRR = varvRR + (RR7-RR3).*(RR4-RR8) + (RR1-RR5).*(RR6-RR2);
   
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate variances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

varmTT  = varmTT./(8*L);
varmDD  = varmDD./(8*L);
varmSS  = varmSS./(8*L);
varmRR  = varmRR./(8*L);

varmzTT  = varmzTT.*(varmzTT > 0)./(4*L);
varmzDD  = varmzDD.*(varmzDD > 0)./(4*L);
varmzSS  = varmzSS.*(varmzSS > 0)./(4*L);
varmzRR  = varmzRR.*(varmzRR > 0)./(4*L);

varfTT  = varfTT.*(varfTT > 0)./(4*L);
varfDD  = varfDD.*(varfDD > 0)./(4*L);
varfSS  = varfSS.*(varfSS > 0)./(4*L);
varfRR  = varfRR.*(varfRR > 0)./(4*L);

varvTT  = varvTT.*(varvTT > 0)./(4*L);
varvDD  = varvDD.*(varvDD > 0)./(4*L);
varvSS  = varvSS.*(varvSS > 0)./(4*L);
varvRR  = varvRR.*(varvRR > 0)./(4*L);