% Write sensitivity results (NSOSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Noise Reduced Reaction-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%    MC COMPUTATION WITH DIMENSIONALITY REDUCTION      %%%%%%%%%%
%%%%%%%%%%%              Authors: Zhang/Goutsias                 %%%%%%%%%%
%%%%%%%%%%%             Last Modified: 09/20/2010                %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write SESIs and JESIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% TIMING

fid = fopen('..\results\reports\timing_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'MC-BASED SPECIES RESULTS (TIMING)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',slambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nSpecies\tSESI\t\tJESI\t\tClassification\n');
for n = 1:nS
    Sens = [spec(n);sigmat(n);etat(n)];
    if sigmat(n) <= theta && etat(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmat(n) > theta && etat(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmat(n) <= theta && etat(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmat(n) > theta && etat(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);

% DURATION

fid = fopen('..\results\reports\duration_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'MC-BASED SPECIES RESULTS (DURATION)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',slambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nSpecies\tSESI\t\tJESI\t\tClassification\n');
for n = 1:nS
    Sens = [spec(n);sigmad(n);etad(n)];
    if sigmad(n) <= theta && etad(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmad(n) > theta && etad(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmad(n) <= theta && etad(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmad(n) > theta && etad(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);

% STRENGTH

fid = fopen('..\results\reports\strength_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'MC-BASED SPECIES RESULTS (STRENGTH)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',slambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nSpecies\tSESI\t\tJESI\t\tClassification\n');
for n = 1:nS
    Sens = [spec(n);sigmas(n);etas(n)];
    if sigmas(n) <= theta && etas(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmas(n) > theta && etas(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmas(n) <= theta && etas(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmas(n) > theta && etas(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);

% RATIO

fid = fopen('..\results\reports\ratio_sens.txt','w');
fprintf(fid,'\t\t\t\n');
fprintf(fid,'MC-BASED SPECIES RESULTS (RATIO)\n\n');
fprintf(fid,'Number of MC-LHS iterations = %1.0f',L);
fprintf(fid,'\n\nSize of biochemical perturbations = %2.4f',slambda);
fprintf(fid,'\n\nSize of experimental perturbations = %2.4f',sigma);
fprintf(fid,'\n\nThreshold = %2.4f',theta);
fprintf(fid,'\n\n\nSpecies\tSESI\t\tJESI\t\tClassification\n');
for n = 1:nS
    Sens = [spec(n);sigmar(n);etar(n)];
    if sigmar(n) <= theta && etar(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\tNON-INFLUENTIAL',Sens);
    end
    if sigmar(n) > theta && etar(n) <= theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (S)',Sens);
    end
    if sigmar(n) <= theta && etar(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (J)',Sens);
    end
    if sigmar(n) > theta && etar(n) > theta
        fprintf(fid,'\n%1.0f\t%10.4f\t%10.4f\t\t****INFLUENTIAL (SJ)',Sens);
    end
end
fclose(fid);