% OHA-based variance estimation (NSOSA-OHA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Noise Reduced Species-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%%                OHA COMPUTATION                      %%%%%%%%%%
%%%%%%%%%%%%            Authors: Zhang/Goutsias                  %%%%%%%%%%
%%%%%%%%%%%%           Last Modified: 09/20/2010                 %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

VqT  = repmat(0,[1,Q]);
VqD  = repmat(0,[1,Q]);
VqS  = repmat(0,[1,Q]);
VqR  = repmat(0,[1,Q]);
VqqT = repmat(0,[Q,Q]); 
VqqD = repmat(0,[Q,Q]);
VqqS = repmat(0,[Q,Q]);
VqqR = repmat(0,[Q,Q]);

for q = 1:Q
    VqT(q) = alphaT1(q)^2 + alphaT2(q)^2;
    VqD(q) = alphaD1(q)^2 + alphaD2(q)^2;
    VqS(q) = alphaS1(q)^2 + alphaS2(q)^2;
    VqR(q) = alphaR1(q)^2 + alphaR2(q)^2;
end    

for q = 1:(Q-1)
    for qq = (q+1):Q
        VqqT(q,qq) = alphaTT1(q,qq)^2 + alphaTT2(q,qq)^2 ...
                   + alphaTT3(q,qq)^2 + alphaTT4(q,qq)^2;
        VqqD(q,qq) = alphaDD1(q,qq)^2 + alphaDD2(q,qq)^2 ...
                   + alphaDD3(q,qq)^2 + alphaDD4(q,qq)^2;
        VqqS(q,qq) = alphaSS1(q,qq)^2 + alphaSS2(q,qq)^2 ...
                   + alphaSS3(q,qq)^2 + alphaSS4(q,qq)^2; 
        VqqR(q,qq) = alphaRR1(q,qq)^2 + alphaRR2(q,qq)^2 ...
                   + alphaRR3(q,qq)^2 + alphaRR4(q,qq)^2;
        VqqT(qq,q) = VqqT(q,qq);
        VqqD(qq,q) = VqqD(q,qq);
        VqqS(qq,q) = VqqS(q,qq);
        VqqR(qq,q) = VqqR(q,qq);
    end
end

for n = 1:N
    VjT(n) = VqT(n) + sum(VqqT(n,(N+1):Q));
    VjD(n) = VqD(n) + sum(VqqD(n,(N+1):Q));
    VjS(n) = VqS(n) + sum(VqqS(n,(N+1):Q));
    VjR(n) = VqR(n) + sum(VqqR(n,(N+1):Q));
    UjT(n) = VqT(n) + sum(VqqT(n,:));
    UjD(n) = VqD(n) + sum(VqqD(n,:));
    UjS(n) = VqS(n) + sum(VqqS(n,:));
    UjR(n) = VqR(n) + sum(VqqR(n,:));
end

VT = sum(VjT) + 0.5*sum(sum(VqqT(1:N,1:N)));
VD = sum(VjD) + 0.5*sum(sum(VqqD(1:N,1:N)));
VS = sum(VjS) + 0.5*sum(sum(VqqS(1:N,1:N)));
VR = sum(VjR) + 0.5*sum(sum(VqqR(1:N,1:N)));